@@ -16,6 +16,7 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"os/exec"
 	"runtime"
 	"sort"
 	"strings"
@@ -38,12 +39,17 @@ const (
 
 // An External is an external source.
 type External struct {
-	Type            ExternalType `json:"type" toml:"type" yaml:"type"`
-	Encrypted       bool         `json:"encrypted" toml:"encrypted" yaml:"encrypted"`
-	Exact           bool         `json:"exact" toml:"exact" yaml:"exact"`
-	Executable      bool         `json:"executable" toml:"executable" yaml:"executable"`
-	StripComponents int          `json:"stripComponents" toml:"stripComponents" yaml:"stripComponents"`
-	URL             string       `json:"url" toml:"url" yaml:"url"`
+	Type       ExternalType `json:"type" toml:"type" yaml:"type"`
+	Encrypted  bool         `json:"encrypted" toml:"encrypted" yaml:"encrypted"`
+	Exact      bool         `json:"exact" toml:"exact" yaml:"exact"`
+	Executable bool         `json:"executable" toml:"executable" yaml:"executable"`
+	Filter     struct {
+		Command string   `json:"command" toml:"command" yaml:"command"`
+		Args    []string `json:"args" toml:"args" yaml:"args"`
+	} `json:"filter" toml:"filter" yaml:"filter"`
+	Format          ArchiveFormat `json:"format" toml:"format" yaml:"format"`
+	StripComponents int           `json:"stripComponents" toml:"stripComponents" yaml:"stripComponents"`
+	URL             string        `json:"url" toml:"url" yaml:"url"`
 }
 
 // A SourceState is a source state.
@@ -225,6 +231,7 @@ func (s *SourceState) Add(sourceSystem System, persistentState PersistentState,
 	sourceUpdates := make([]sourceUpdate, 0, len(destAbsPathInfos))
 	newSourceStateEntries := make(map[SourceRelPath]SourceStateEntry)
 	newSourceStateEntriesByTargetRelPath := make(map[RelPath]SourceStateEntry)
+	nonEmptyDirs := make(map[SourceRelPath]struct{})
 DESTABSPATH:
 	for _, destAbsPath := range destAbsPaths {
 		destAbsPathInfo := destAbsPathInfos[destAbsPath]
@@ -244,6 +251,7 @@ DESTABSPATH:
 		} else {
 			return fmt.Errorf("%s: parent directory not in source state", destAbsPath)
 		}
+		nonEmptyDirs[parentSourceRelPath] = struct{}{}
 
 		actualStateEntry, err := NewActualStateEntry(destSystem, destAbsPath, destAbsPathInfo, nil)
 		if err != nil {
@@ -302,25 +310,33 @@ DESTABSPATH:
 		newSourceStateEntriesByTargetRelPath[targetRelPath] = newSourceStateEntry
 
 		sourceUpdates = append(sourceUpdates, update)
+	}
 
-		if _, ok := newSourceStateEntry.(*SourceStateDir); ok {
-			dotKeepFileRelPath := sourceEntryRelPath.Join(NewSourceRelPath(".keep"))
+	// Create .keep files in empty added directories.
+	for sourceEntryRelPath, sourceStateEntry := range newSourceStateEntries {
+		if _, ok := sourceStateEntry.(*SourceStateDir); !ok {
+			continue
+		}
+		if _, ok := nonEmptyDirs[sourceEntryRelPath]; ok {
+			continue
+		}
 
-			dotKeepFileSourceUpdate := sourceUpdate{
-				entryState: &EntryState{
-					Type: EntryStateTypeFile,
-					Mode: 0o666 &^ s.umask,
-				},
-				sourceRelPaths: []SourceRelPath{dotKeepFileRelPath},
-			}
-			sourceUpdates = append(sourceUpdates, dotKeepFileSourceUpdate)
+		dotKeepFileRelPath := sourceEntryRelPath.Join(NewSourceRelPath(".keep"))
 
-			newSourceStateEntries[dotKeepFileRelPath] = &SourceStateFile{
-				targetStateEntry: &TargetStateFile{
-					empty: true,
-					perm:  0o666 &^ s.umask,
-				},
-			}
+		dotKeepFileSourceUpdate := sourceUpdate{
+			entryState: &EntryState{
+				Type: EntryStateTypeFile,
+				Mode: 0o666 &^ s.umask,
+			},
+			sourceRelPaths: []SourceRelPath{dotKeepFileRelPath},
+		}
+		sourceUpdates = append(sourceUpdates, dotKeepFileSourceUpdate)
+
+		newSourceStateEntries[dotKeepFileRelPath] = &SourceStateFile{
+			targetStateEntry: &TargetStateFile{
+				empty: true,
+				perm:  0o666 &^ s.umask,
+			},
 		}
 	}
 
@@ -894,7 +910,15 @@ func (s *SourceState) TemplateData() map[string]interface{} {
 
 // addExternal adds external source entries to s.
 func (s *SourceState) addExternal(sourceAbsPath AbsPath) error {
-	_, name := sourceAbsPath.Split()
+	parentAbsPath, name := sourceAbsPath.Split()
+
+	parentRelPath, err := parentAbsPath.TrimDirPrefix(s.sourceDirAbsPath)
+	if err != nil {
+		return err
+	}
+	parentSourceRelPath := NewSourceRelDirPath(parentRelPath)
+	parentTargetSourceRelPath := parentSourceRelPath.TargetRelPath(s.encryption.EncryptedSuffix())
+
 	suffix := mustTrimPrefix(string(name), externalName+".")
 	format, ok := Formats[suffix]
 	if !ok {
@@ -904,9 +928,17 @@ func (s *SourceState) addExternal(sourceAbsPath AbsPath) error {
 	if err != nil {
 		return fmt.Errorf("%s: %w", sourceAbsPath, err)
 	}
-	if err := format.Unmarshal(data, &s.externals); err != nil {
+	externals := make(map[RelPath]External)
+	if err := format.Unmarshal(data, &externals); err != nil {
 		return fmt.Errorf("%s: %w", sourceAbsPath, err)
 	}
+	for relPath, external := range externals {
+		targetRelPath := parentTargetSourceRelPath.Join(relPath)
+		if _, ok := s.externals[targetRelPath]; ok {
+			return fmt.Errorf("%s: duplicate externals", targetRelPath)
+		}
+		s.externals[targetRelPath] = external
+	}
 	return nil
 }
 
@@ -1027,21 +1059,13 @@ func (s *SourceState) executeTemplate(templateAbsPath AbsPath) ([]byte, error) {
 	return s.ExecuteTemplateData(string(templateAbsPath), data)
 }
 
-// getExternalData reads the external data for externalRelPath from
-// external.URL.
-func (s *SourceState) getExternalData(ctx context.Context, externalRelPath RelPath, external External, options *ReadOptions) ([]byte, error) {
+func (s *SourceState) getExternalDataRaw(ctx context.Context, externalRelPath RelPath, external External, options *ReadOptions) ([]byte, error) {
 	cacheKey := hex.EncodeToString(SHA256Sum([]byte(external.URL)))
 	cachedDataAbsPath := s.cacheDirAbsPath.Join("external", RelPath(cacheKey))
 	if options == nil || !options.RefreshExternals {
 		data, err := s.system.ReadFile(cachedDataAbsPath)
 		switch {
 		case err == nil:
-			if external.Encrypted {
-				data, err = s.encryption.Decrypt(data)
-				if err != nil {
-					return nil, fmt.Errorf("%s: %s: %w", externalRelPath, external.URL, err)
-				}
-			}
 			return data, nil
 		case !errors.Is(err, fs.ErrNotExist):
 			return nil, err
@@ -1081,6 +1105,17 @@ func (s *SourceState) getExternalData(ctx context.Context, externalRelPath RelPa
 		return nil, err
 	}
 
+	return data, nil
+}
+
+// getExternalDataRaw reads the external data for externalRelPath from
+// external.URL.
+func (s *SourceState) getExternalData(ctx context.Context, externalRelPath RelPath, external External, options *ReadOptions) ([]byte, error) {
+	data, err := s.getExternalDataRaw(ctx, externalRelPath, external, options)
+	if err != nil {
+		return nil, err
+	}
+
 	if external.Encrypted {
 		data, err = s.encryption.Decrypt(data)
 		if err != nil {
@@ -1088,6 +1123,16 @@ func (s *SourceState) getExternalData(ctx context.Context, externalRelPath RelPa
 		}
 	}
 
+	if external.Filter.Command != "" {
+		//nolint:gosec
+		cmd := exec.Command(external.Filter.Command, external.Filter.Args...)
+		cmd.Stdin = bytes.NewReader(data)
+		data, err = s.system.IdempotentCmdOutput(cmd)
+		if err != nil {
+			return nil, fmt.Errorf("%s: %s: %w", externalRelPath, external.URL, err)
+		}
+	}
+
 	return data, nil
 }
 
@@ -1354,6 +1399,7 @@ func (s *SourceState) newSourceStateDirEntry(info fs.FileInfo, parentSourceRelPa
 		TargetName: info.Name(),
 		Exact:      options.Exact,
 		Private:    isPrivate(info),
+		ReadOnly:   isReadOnly(info),
 	}
 	sourceRelPath := parentSourceRelPath.Join(NewSourceRelDirPath(RelPath(dirAttr.SourceName())))
 	return &SourceStateDir{
@@ -1377,6 +1423,7 @@ func (s *SourceState) newSourceStateFileEntryFromFile(actualStateFile *ActualSta
 		Encrypted:  options.Encrypt,
 		Executable: isExecutable(info),
 		Private:    isPrivate(info),
+		ReadOnly:   isReadOnly(info),
 		Template:   options.Template,
 	}
 	if options.Create {
@@ -1497,7 +1544,8 @@ func (s *SourceState) readExternalArchive(ctx context.Context, externalRelPath R
 		externalRelPath: {
 			&SourceStateDir{
 				Attr: DirAttr{
-					Exact: external.Exact,
+					TargetName: externalRelPath.Base(),
+					Exact:      external.Exact,
 				},
 				sourceRelPath: sourceRelPath,
 				targetStateEntry: &TargetStateDir{
@@ -1507,7 +1555,12 @@ func (s *SourceState) readExternalArchive(ctx context.Context, externalRelPath R
 		},
 	}
 
-	if err := walkArchive(path, data, func(name string, info fs.FileInfo, r io.Reader, linkname string) error {
+	format := external.Format
+	if format == ArchiveFormatUnknown {
+		format = GuessArchiveFormat(path, data)
+	}
+
+	if err := walkArchive(data, format, func(name string, info fs.FileInfo, r io.Reader, linkname string) error {
 		if external.StripComponents > 0 {
 			components := strings.Split(name, "/")
 			if len(components) <= external.StripComponents {
@@ -1532,7 +1585,10 @@ func (s *SourceState) readExternalArchive(ctx context.Context, externalRelPath R
 			}
 			sourceStateEntry = &SourceStateDir{
 				Attr: DirAttr{
-					Exact: external.Exact,
+					TargetName: info.Name(),
+					Exact:      external.Exact,
+					Private:    isPrivate(info),
+					ReadOnly:   isReadOnly(info),
 				},
 				sourceRelPath:    sourceRelPath,
 				targetStateEntry: targetStateEntry,
@@ -1544,8 +1600,12 @@ func (s *SourceState) readExternalArchive(ctx context.Context, externalRelPath R
 			}
 			lazyContents := newLazyContents(contents)
 			fileAttr := FileAttr{
-				Empty:      true,
-				Executable: info.Mode().Perm()&0o111 != 0,
+				TargetName: info.Name(),
+				Type:       SourceFileTypeFile,
+				Empty:      info.Size() == 0,
+				Executable: isExecutable(info),
+				Private:    isPrivate(info),
+				ReadOnly:   isReadOnly(info),
 			}
 			targetStateEntry := &TargetStateFile{
 				lazyContents: lazyContents,
@@ -1553,6 +1613,7 @@ func (s *SourceState) readExternalArchive(ctx context.Context, externalRelPath R
 				perm:         fileAttr.perm() &^ s.umask,
 			}
 			sourceStateEntry = &SourceStateFile{
+				lazyContents:     lazyContents,
 				Attr:             fileAttr,
 				sourceRelPath:    sourceRelPath,
 				targetStateEntry: targetStateEntry,
@@ -1562,6 +1623,10 @@ func (s *SourceState) readExternalArchive(ctx context.Context, externalRelPath R
 				lazyLinkname: newLazyLinkname(linkname),
 			}
 			sourceStateEntry = &SourceStateFile{
+				Attr: FileAttr{
+					TargetName: info.Name(),
+					Type:       SourceFileTypeSymlink,
+				},
 				sourceRelPath:    sourceRelPath,
 				targetStateEntry: targetStateEntry,
 			}
