@@ -4,6 +4,8 @@ import (
 	"context"
 	"errors"
 	"io/fs"
+	"net/http"
+	"net/http/httptest"
 	"path/filepath"
 	"testing"
 	"text/template"
@@ -179,6 +181,24 @@ func TestSourceStateAdd(t *testing.T) {
 				),
 			},
 		},
+		{
+			name: "dir_readonly_unix",
+			destAbsPaths: []AbsPath{
+				"/home/user/.readonly_dir",
+			},
+			addOptions: AddOptions{
+				Include: NewEntryTypeSet(EntryTypesAll),
+			},
+			extraRoot: map[string]interface{}{
+				"/home/user/.readonly_dir": &vfst.Dir{Perm: 0o555},
+			},
+			tests: []interface{}{
+				vfst.TestPath("/home/user/.local/share/chezmoi/readonly_dot_readonly_dir",
+					vfst.TestIsDir,
+					vfst.TestModePerm(0o777&^chezmoitest.Umask),
+				),
+			},
+		},
 		{
 			name: "empty",
 			destAbsPaths: []AbsPath{
@@ -348,6 +368,28 @@ func TestSourceStateAdd(t *testing.T) {
 				),
 			},
 		},
+		{
+			name: "file_readonly_unix",
+			destAbsPaths: []AbsPath{
+				"/home/user/.readonly",
+			},
+			addOptions: AddOptions{
+				Include: NewEntryTypeSet(EntryTypesAll),
+			},
+			extraRoot: map[string]interface{}{
+				"/home/user/.readonly": &vfst.File{
+					Perm:     0o444,
+					Contents: []byte("# contents of .readonly\n"),
+				},
+			},
+			tests: []interface{}{
+				vfst.TestPath("/home/user/.local/share/chezmoi/readonly_dot_readonly",
+					vfst.TestModeIsRegular,
+					vfst.TestModePerm(0o666&^chezmoitest.Umask),
+					vfst.TestContentsString("# contents of .readonly\n"),
+				),
+			},
+		},
 		{
 			name: "symlink",
 			destAbsPaths: []AbsPath{
@@ -1187,6 +1229,73 @@ func TestSourceStateRead(t *testing.T) {
 	}
 }
 
+func TestSourceStateReadExternal(t *testing.T) {
+	httpServer := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		_, err := w.Write([]byte("data"))
+		require.NoError(t, err)
+	}))
+	defer httpServer.Close()
+
+	for _, tc := range []struct {
+		name              string
+		root              interface{}
+		expectedExternals map[RelPath]External
+	}{
+		{
+			name: "external",
+			root: map[string]interface{}{
+				"/home/user/.local/share/chezmoi": map[string]interface{}{
+					".chezmoiexternal.yaml": chezmoitest.JoinLines(
+						`file:`,
+						`    type: "file"`,
+						`    url: "`+httpServer.URL+`/file"`,
+					),
+				},
+			},
+			expectedExternals: map[RelPath]External{
+				"file": {
+					Type: "file",
+					URL:  httpServer.URL + "/file",
+				},
+			},
+		},
+		{
+			name: "external_in_subdir",
+			root: map[string]interface{}{
+				"/home/user/.local/share/chezmoi/dot_dir": map[string]interface{}{
+					".chezmoiexternal.yaml": chezmoitest.JoinLines(
+						`file:`,
+						`    type: "file"`,
+						`    url: "`+httpServer.URL+`/file"`,
+					),
+				},
+			},
+			expectedExternals: map[RelPath]External{
+				".dir/file": {
+					Type: "file",
+					URL:  httpServer.URL + "/file",
+				},
+			},
+		},
+	} {
+		t.Run(tc.name, func(t *testing.T) {
+			chezmoitest.WithTestFS(t, tc.root, func(fileSystem vfs.FS) {
+				ctx := context.Background()
+				system := NewRealSystem(fileSystem)
+				s := NewSourceState(
+					WithBaseSystem(system),
+					WithCacheDir("/home/user/.cache/chezmoi"),
+					WithDestDir("/home/user"),
+					WithSourceDir("/home/user/.local/share/chezmoi"),
+					WithSystem(system),
+				)
+				require.NoError(t, s.Read(ctx, nil))
+				assert.Equal(t, tc.expectedExternals, s.externals)
+			})
+		})
+	}
+}
+
 func TestSourceStateTargetRelPaths(t *testing.T) {
 	for _, tc := range []struct {
 		name                   string
