@@ -1,7 +1,16 @@
 package chezmoi
 
+import (
+	"encoding/hex"
+
+	"github.com/rs/zerolog"
+
+	"github.com/twpayne/chezmoi/v2/internal/chezmoilog"
+)
+
 // A SourceStateEntry represents the state of an entry in the source state.
 type SourceStateEntry interface {
+	zerolog.LogObjectMarshaler
 	Evaluate() error
 	Order() int
 	SourceRelPath() SourceRelPath
@@ -42,6 +51,12 @@ func (s *SourceStateDir) Evaluate() error {
 	return nil
 }
 
+// MarshalZerologObject implements zerolog.LogObjectMarshaler.
+func (s *SourceStateDir) MarshalZerologObject(e *zerolog.Event) {
+	e.Stringer("sourceRelPath", s.sourceRelPath)
+	e.Object("attr", s.Attr)
+}
+
 // Order returns s's order.
 func (s *SourceStateDir) Order() int {
 	return 0
@@ -63,6 +78,23 @@ func (s *SourceStateFile) Evaluate() error {
 	return err
 }
 
+// MarshalZerologObject implements zerolog.LogObjectMarshaler.
+func (s *SourceStateFile) MarshalZerologObject(e *zerolog.Event) {
+	e.Stringer("sourceRelPath", s.sourceRelPath)
+	e.Interface("attr", s.Attr)
+	contents, contentsErr := s.Contents()
+	e.Bytes("contents", chezmoilog.FirstFewBytes(contents))
+	if contentsErr != nil {
+		e.Str("contentsErr", contentsErr.Error())
+	}
+	e.Err(contentsErr)
+	contentsSHA256, contentsSHA256Err := s.ContentsSHA256()
+	e.Str("contentsSHA256", hex.EncodeToString(contentsSHA256))
+	if contentsSHA256Err != nil {
+		e.Str("contentsSHA256Err", contentsSHA256Err.Error())
+	}
+}
+
 // Order returns s's order.
 func (s *SourceStateFile) Order() int {
 	return s.Attr.Order
@@ -87,6 +119,11 @@ func (s *SourceStateRemove) Evaluate() error {
 	return nil
 }
 
+// MarshalZerologObject implements zerolog.LogObjectMarshaler.
+func (s *SourceStateRemove) MarshalZerologObject(e *zerolog.Event) {
+	e.Stringer("targetRelPath", s.targetRelPath)
+}
+
 // Order returns s's order.
 func (s *SourceStateRemove) Order() int {
 	return 0
@@ -107,6 +144,12 @@ func (s *SourceStateRenameDir) Evaluate() error {
 	return nil
 }
 
+// MarashalLogObject implements zerolog.LogObjectMarshaler.
+func (s *SourceStateRenameDir) MarshalZerologObject(e *zerolog.Event) {
+	e.Stringer("oldSourceRelPath", s.oldSourceRelPath)
+	e.Stringer("newSourceRelPath", s.newSourceRelPath)
+}
+
 // Order returns s's order.
 func (s *SourceStateRenameDir) Order() int {
 	return -1
