@@ -13,6 +13,7 @@ import (
 	"strings"
 
 	"github.com/charmbracelet/glamour"
+	"github.com/rs/zerolog"
 	"github.com/spf13/cobra"
 	"go.etcd.io/bbolt"
 
@@ -83,6 +84,13 @@ func init() {
 	}
 }
 
+func (v VersionInfo) MarshalZerologObject(e *zerolog.Event) {
+	e.Str("version", v.Version)
+	e.Str("commit", v.Commit)
+	e.Str("date", v.Date)
+	e.Str("builtBy", v.BuiltBy)
+}
+
 // Main runs chezmoi and returns an exit code.
 func Main(versionInfo VersionInfo, args []string) int {
 	if err := runMain(versionInfo, args); err != nil {
