@@ -294,7 +294,6 @@ func newConfig(options ...configOption) (*Config, error) {
 		},
 		archive: archiveCmdConfig{
 			exclude:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
-			format:    archiveFormatTar,
 			include:   chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
 			recursive: true,
 		},
@@ -721,7 +720,7 @@ func (c *Config) defaultTemplateData() map[string]interface{} {
 	}
 
 	var osRelease map[string]interface{}
-	if rawOSRelease, err := chezmoi.OSRelease(c.fileSystem); err == nil {
+	if rawOSRelease, err := chezmoi.OSRelease(c.baseSystem); err == nil {
 		osRelease = upperSnakeCaseToCamelCaseMap(rawOSRelease)
 	} else {
 		log.Debug().
@@ -1060,18 +1059,21 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 
 	persistentFlags.Var(&c.Color, "color", "Colorize output")
 	persistentFlags.VarP(&c.DestDirAbsPath, "destination", "D", "Set destination directory")
+	persistentFlags.Var(&c.Mode, "mode", "Mode")
 	persistentFlags.BoolVar(&c.Safe, "safe", c.Safe, "Safely replace files and symlinks")
 	persistentFlags.VarP(&c.SourceDirAbsPath, "source", "S", "Set source directory")
-	persistentFlags.Var(&c.Mode, "mode", "Mode")
 	persistentFlags.Var(&c.UseBuiltinAge, "use-builtin-age", "Use builtin age")
 	persistentFlags.Var(&c.UseBuiltinGit, "use-builtin-git", "Use builtin git")
-	for _, key := range []string{
-		"color",
-		"destination",
-		"mode",
-		"source",
+	for viperKey, key := range map[string]string{
+		"color":         "color",
+		"destDir":       "destination",
+		"mode":          "mode",
+		"safe":          "safe",
+		"sourceDir":     "source",
+		"useBuiltinAge": "use-builtin-age",
+		"useBuiltinGit": "use-builtin-git",
 	} {
-		if err := viper.BindPFlag(key, persistentFlags.Lookup(key)); err != nil {
+		if err := viper.BindPFlag(viperKey, persistentFlags.Lookup(key)); err != nil {
 			return nil, err
 		}
 	}
@@ -1468,19 +1470,18 @@ func (c *Config) promptChoice(prompt string, choices []string) (string, error) {
 }
 
 func (c *Config) readConfig() error {
-	v := viper.New()
-	v.SetConfigFile(string(c.configFileAbsPath))
+	viper.SetConfigFile(string(c.configFileAbsPath))
 	if c.configFormat != "" {
-		v.SetConfigType(c.configFormat.String())
+		viper.SetConfigType(c.configFormat.String())
 	}
-	v.SetFs(afero.FromIOFS{FS: c.fileSystem})
-	switch err := v.ReadInConfig(); {
+	viper.SetFs(afero.FromIOFS{FS: c.fileSystem})
+	switch err := viper.ReadInConfig(); {
 	case errors.Is(err, fs.ErrNotExist):
 		return nil
 	case err != nil:
 		return err
 	}
-	if err := v.Unmarshal(c, viperDecodeConfigOptions...); err != nil {
+	if err := viper.Unmarshal(c, viperDecodeConfigOptions...); err != nil {
 		return err
 	}
 	return c.validateData()
